@@ -1,31 +1,5 @@
-use bytes::{Buf, BufMut, Bytes};
-use std::error;
-use std::fmt;
-
-#[derive(Debug)]
-pub enum ErrorError {
-    InvalidValueChar,
-    InvalidFirstChar,
-    InvalidTerminate,
-}
-
-impl fmt::Display for ErrorError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match self {
-            ErrorError::InvalidValueChar => {
-                write!(f, "[ErrorError] Invalid value char.")
-            }
-            ErrorError::InvalidFirstChar => {
-                write!(f, "[ErrorError] Invalid first char.")
-            }
-            ErrorError::InvalidTerminate => {
-                write!(f, "[ErrorError] Invalid terminate.")
-            }
-        }
-    }
-}
-
-impl error::Error for ErrorError {}
+use crate::RespError;
+use bytes::{Buf, BufMut, Bytes, BytesMut};
 
 #[derive(Debug, Clone, PartialEq)]
 pub struct Error(Bytes);
@@ -41,14 +15,13 @@ impl Error {
     /// let error = Error::new(b"Invalid type.");
     /// ```
     #[inline]
-    pub fn new(input: &[u8]) -> Error {
-        let mut vector = Vec::with_capacity(input.len() + 3);
-        vector.put_u8(0x2d); // "-"
-        vector.put_slice(input);
-        vector.put_u8(0x0d); // CR
-        vector.put_u8(0x0a); // LF
-        let bytes = Bytes::from(vector);
-        Error(bytes)
+    pub fn new(input: &[u8]) -> Self {
+        let mut bytes = BytesMut::with_capacity(input.len() + 3);
+        bytes.put_u8(0x2d); // "-"
+        bytes.put_slice(input);
+        bytes.put_u8(0x0d); // CR
+        bytes.put_u8(0x0a); // LF
+        Self::from_bytes(bytes.freeze())
     }
 
     #[inline]
@@ -73,14 +46,14 @@ impl Error {
         vector
     }
 
-    pub fn validate_value(input: &[u8]) -> Result<(), ErrorError> {
+    pub fn validate_value(input: &[u8]) -> Result<(), RespError> {
         let mut index = 0;
         let length = input.len();
         while index < length && input[index] != 0x0d && input[index] != 0x0a {
             index += 1;
         }
         if index != length {
-            return Err(ErrorError::InvalidValueChar);
+            return Err(RespError::InvalidValue);
         }
         Ok(())
     }
@@ -103,19 +76,25 @@ impl Error {
         Self::from_bytes(bytes)
     }
 
-    pub fn parse(input: &[u8], start: &mut usize, end: &usize) -> Result<Error, ErrorError> {
+    pub fn while_valid(input: &[u8], start: &mut usize, end: &usize) -> Result<(), RespError> {
         let mut index = *start;
         if index >= *end || input[index] != 0x2d {
-            return Err(ErrorError::InvalidFirstChar);
+            return Err(RespError::InvalidFirstChar);
         }
         index += 1;
         while index < *end && input[index] != 0x0d && input[index] != 0x0a {
             index += 1;
         }
         if index + 1 >= *end || input[index] != 0x0d || input[index + 1] != 0x0a {
-            return Err(ErrorError::InvalidTerminate);
+            return Err(RespError::InvalidTerminate);
         }
-        index += 2;
+        *start = index + 2;
+        Ok(())
+    }
+
+    pub fn parse(input: &[u8], start: &mut usize, end: &usize) -> Result<Self, RespError> {
+        let mut index = *start;
+        Self::while_valid(input, &mut index, end)?;
         let value = Self::from_slice(&input[*start..index]);
         *start = index;
         Ok(value)
@@ -155,7 +134,7 @@ mod tests_error {
     }
 
     #[test]
-    #[should_panic(expected = "InvalidValueChar")]
+    #[should_panic(expected = "InvalidValue")]
     fn test_validate_invalid_value() {
         let value = b"Error\r\n message";
         assert_eq!(Error::validate_value(value).unwrap(), ())
